

from signal_generation import generator
from trace_checker import formula_generator
from trace_checker import STL
from optimization import grid_search
import plot
from collections import namedtuple
import scipy.io

FormulaValuation = namedtuple('FormulaValuation', ['formula', 'valuation'])


def save_formula_search_results(folder_name, results, best_formulas, result_file):
  """

  Args:
    folder_name:
  """
  file_name = folder_name + 'formula_search_results' + result_file
  r = dict()
  r['results'] = results
  r['best_formulas'] = best_formulas
  scipy.io.savemat(file_name, r)



def formula_search(metric_list, operator_counts, parameter_domains, folder_name, trace_count, generate_signals, signal_file_base,
                   process_count, save, return_type, result_file):
  """

  Args:
    metric_list
    operator_counts:
    parameter_domains:
    folder_name:
    trace_count:
    generate_signals:
    signal_file_base:
    process_count:

  Returns:

  """

  # If needed, generate signals.
  if generate_signals == "GAUS":
    generator.generate_signals(folder_name, signal_file_base, trace_count, process_count)
  elif generate_signals == "PUSH":
    generator.generate_signals_push(folder_name, signal_file_base, trace_count, process_count)

  # Perform the search for all given formula counts.
  results = {oc: [] for oc in operator_counts}
  best_formulas = {oc: [] for oc in operator_counts}
  mismatch_count_bound = 100000000000
  for oc in operator_counts:
    results[oc], best_formulas[oc], mcb = formula_search_operator_count(metric_list, oc, parameter_domains, folder_name,
                                                trace_count, signal_file_base, process_count, return_type,
                                                                   mismatch_count_bound)

    mismatch_count_bound = min(mcb, mismatch_count_bound)
    print "Results for oc %d " % oc
    print best_formulas[oc]
    if mismatch_count_bound == 0:
      break

  print "-----------"
  print best_formulas
  if save:
    save_formula_search_results(folder_name, results, best_formulas, result_file)
  return results, best_formulas

def formula_search_operator_count(metric_list, oc, parameter_domains, folder_name, trace_count, signal_file_base, process_count,
                                  return_type, mismatch_count_bound):
  """ For the given operator count, compute the number of template formulas and
      run grid search for each template formula.
  """
  template_formula_list = formula_generator.generate_formula_tree(metric_list, oc, return_formula_string=True)

  print "----------- For operator count " + str(oc) + ", " + str(len(template_formula_list)) + " formulas will be tested."
  # Process and remove the formulas with double Since.
  template_formula_list = [formula for formula in template_formula_list if formula.split().count('S') < 2]
  print "----------- For operator count " + str(oc) + ", " + str(len(template_formula_list)) + " formulas will be tested."


  # For each template formula, run a grid search. Store the results, remember the best one.
  results = {f:{} for f in template_formula_list}
  best_formula = FormulaValuation(formula='False', valuation=0)
  if return_type == 1:
    best_formula = FormulaValuation(formula='False', valuation=10000000000000)
  mcb_local = mismatch_count_bound
  for template_formula in template_formula_list:
    formula, parameter_domains_in_formula = generate_formula_from_template(template_formula, parameter_domains)
    results[template_formula] = parameter_search_for_formula(formula, parameter_domains_in_formula, folder_name,
                                                             trace_count, signal_file_base, process_count, return_type,
                                                             mcb_local)


    if return_type == 0 and results[template_formula].valuation > best_formula.valuation:
      best_formula = results[template_formula]
    if return_type == 1 and results[template_formula].valuation < best_formula.valuation:
      best_formula = results[template_formula]
      if results[template_formula].valuation < mcb_local:
        print "UPDATE BEST:" + best_formula.formula + " "  +  str(best_formula.valuation)
        if best_formula.valuation==0:
          break # No need to check the rest
      mcb_local = min(mcb_local, results[template_formula].valuation)
      # TODO continue from here update the return statement
  return results, best_formula, mcb_local

def parameter_search_for_formula(formula, parameter_domains_for_formula, folder_name, trace_count, signal_file_base,
                                 process_count, return_type, mismatch_count_bound):
  parameter_list = parameter_domains_for_formula.keys()
  parameter_domain = [parameter_domains_for_formula[pa] for pa in parameter_list]

  prefix_formula = STL.infix_to_prefix(formula)
  best_v, params, all_results, time_passed = grid_search.grid_search(
    prefix_formula, parameter_list, parameter_domain, folder_name, signal_file_base, trace_count, '', process_count,
    return_type, mismatch_count_bound)

  #result_file = folder_name + "".join(formula.split()) + ".mat"
  #_ = plot.convert_save_results(parameter_list, parameter_domain, all_results, result_file, formula,
  #                                                  time_passed, trace_count, process_count, best_v, params)

  formula_n = formula
  for p, v in zip(parameter_list, params):
    formula_n = formula_n.replace(p, str(v))
  return FormulaValuation(formula=formula_n, valuation=best_v)





def generate_formula_from_template(template_formula, parameter_domains):

  # first process the formula:
  formula_tokens = template_formula.split()
  # tokens that start with p and has 2 characters:
  parameters  = [p for p in formula_tokens if p[0] == 'p' and len(p) == 2]
  parameters_set = list(set(parameters)) # make unique

  parameter_domains_in_formula = {}
  new_formula = template_formula
  for p in parameters_set:
    for i in xrange(parameters.count(p)):
      p_new = p + str(i)
      for j in xrange(len(formula_tokens)):
        if formula_tokens[j] == p:
          formula_tokens[j] = p_new
          parameter_domains_in_formula[p_new] = parameter_domains[p]
          break
      #new_formula = new_formula.replace(p, p_new, 1)

  new_formula = " ".join(formula_tokens)
  print new_formula
  return new_formula, parameter_domains_in_formula



