

import itertools
import evaluator

import trace_checker.STL as STL
from timeit import default_timer as timer
from multiprocessing import Pool
from functools import partial




def evaluate_signals(formula, folder_name, signal_file_base, sfc, sfr, ret_type=0, mismatch_count=0):
  """

  Args:
    formula:
    folder_name:
    signal_file_base:
    sfc:
    sfr:
    ret_type:  Return type, 0: return average, 1: return number of mismatches.
    mismatch_count: When ret_type is 1, stop computation if mismatch count is greater than mismatch_count

  Returns:

  """

  count = 0
  total = 0
  stn = STL.SyntaxTreeNode()
  stn.initialize_node(formula.split(), 0)
  # max_r = 0

  for i in xrange(sfc):
    s_file = folder_name + signal_file_base + '_' +str(i) + sfr
    label_file =  s_file + "_label"
    # The result is average hit (0,1) if ret_type = 0, it is the number of mismatches if ret_type = 1
    r = evaluator.evaluate(formula, s_file, label_file, stn, ret_type, mismatch_count)
    #max_r = max(max_r, r)
    count += 1
    total += r # total number of mismatches for ret_type = 1
    if ret_type==1 and total > mismatch_count: # Total mismatch count, if it is higher, stop the computation.
      return  total # a lower bound on the number of mismatches, it can increase.


  # print 'AVERAGE: ' + str(total/count)
  if ret_type == 0:
    return total/count
  else:
    return total



def evaluate_signals_from_param(params, parameter_list, formula, folder_name, signal_file_base, signal_file_count,
                                signal_file_rest, return_type, mismatch_count_bound):

  tmp_formula = formula
  for p, v in zip(parameter_list, params):
    tmp_formula = tmp_formula.replace(p, str(v))

  return evaluate_signals(tmp_formula, folder_name, signal_file_base, signal_file_count, signal_file_rest,
                          return_type, mismatch_count_bound), params


def my_test(t, t2, t3):
  print 'my test ' + str(t) + ' ' + str(t2) + ' ' + str(t3)
  return t*t


def multip():
  p = Pool(2)
  m = partial(my_test, t2=2, t3=12)
  t = [0, 1, 2, 3, 4, 5]
  r = p.map(m, t)
  print r


def grid_search(formula, parameter_list, parameter_domain, folder_name, signal_file_base, signal_file_count,
                signal_file_rest, mp, return_type=0, mismatch_count_bound=1000000000):
  """

  Args:
    formula: Formula, e.g., A 0 p1 P 0 p2 x > 10
    parameter_list: A list of strings, [p1, p2]
    parameter_domain: A list of lists representing domains for each parameter, [[1,2], [1,2,3,4]]
    folder_name: A string, the files will be read from here.
    signal_file_base: A string,
    signal_file_count: A string, folder_name + signal_file_base + i + signal_file_rest, i< signal_file_count files will be loaded.
    signal_file_rest: A string
    mp: An integer, process count.
    return_type: 0 for average number of matches, 1 for the number of mismatches
    mismatch_count_bound: Stop the computation is the mismatch_count is higher than this one.
  Returns:
    A list of parameters giving the optimal valuation and the average valuation.
  """

  best_val = 0
  best_params = None
  start = timer()

  if mp > 0:

    partial_evaluate_signals = partial(evaluate_signals_from_param, parameter_list=parameter_list, formula=formula,
                                       folder_name=folder_name, signal_file_base=signal_file_base,
                                       signal_file_count=signal_file_count, signal_file_rest=signal_file_rest,
                                       return_type=return_type, mismatch_count_bound=mismatch_count_bound)
    pool = Pool(processes=mp)
    results = pool.map(partial_evaluate_signals, itertools.product(*parameter_domain))
    pool.close()
    pool.join()
    if return_type==0:# average hit, maximization
      r = max(results, key=lambda item:item[0])
    else: # the number of mismatches, minimization
      r = min(results, key=lambda  item:item[0])
    end = timer()
    tp = end - start
    print "RESULT: ",  formula, " " , r[0], r[1], tp
    return r[0], r[1], results, tp

  else:

    for params in itertools.product(*parameter_domain):
      tmp_formula = formula
      for p, v in zip(parameter_list, params):
        tmp_formula = tmp_formula.replace(p, str(v))

      v = evaluate_signals(tmp_formula, folder_name, signal_file_base, signal_file_count, signal_file_rest)
      if v > best_val:
        best_params = params
        best_val = v

  end = timer()
  print (end - start)
  return best_val, best_params, None

