"""
Signal generator.
"""

from collections import namedtuple
import random
import sys
from plot import plot


Options = namedtuple('Options', ["duration", "violate", "min_v_dur", "max_v_dur", "v_dur", "min_step", "max_step",
                                 "sleep", "s_min", "s_max", "t"])
# duration: signal duration  (floating point)
# violate: violation occurs or not (boolean)
# min_v_dur: minimum violation duration
# max_v_dur: maximum violation duration
# v_dur: min_v_dur <= v_dur <= max_v_dur, [ v_dur, max_v_dur] will be labeled as violating
# min_step
# max_step
# sleep

GaussOptions = namedtuple('GaussOptions', ["v_diff", "mean", "deviation"])


def signal_generator(index, folder_name, name, s_type, options, additional_options):
  """
  Generate the signal and store it in a file according to given options.

  index: index of the signal
  name: signal name, e.g. metric name
  type: type of the generator
  options: generator options an instance of Options
  additional_options: additional options for generator type
  """
  f_signal_name = folder_name + "/" + name + "_" + str(index) # + "_" + str(options.t)
  f_label_name = f_signal_name + "_label"
  f_s = open(f_signal_name, 'w')
  f_l = open(f_label_name, 'w')

  if s_type == "GAUSS":
    gauss_signal_generator(f_s, f_l, options, additional_options)
  elif s_type == "PUSH":
    push_signal_generator(f_s, f_l, options)
  else:
    print "Unknown signal type:" + s_type

  f_s.close()
  f_l.close()
  print "Done %d" % index


  return


def push_signal_generator(f_s, f_l, options):
  passed_time = 0

  s = [0, 0]
  if options.violate:
    v_dur = random.uniform(options.min_v_dur, options.max_v_dur/2 )
    v_start_time = random.uniform(0, options.duration - v_dur)
    v_end_time = v_start_time + v_dur

    if v_start_time > options.duration/2:
      push_start_time = random.uniform(0, options.duration/2)
    else:
      push_start_time = random.uniform(options.duration/2, options.duration - v_dur)
    push_end_time = push_start_time + random.uniform(options.min_v_dur, options.max_v_dur)

  viol_rate = random.uniform(15, 25)
  with_push = random.uniform(20, 30)

  while passed_time < options.duration:

    s[0] = 0 # no push
    s[1] = 0 # no restart
    l = 0# no violation
    if random.uniform(0, 1) < 0.04:
      s[1] = random.uniform(0, 10) # no violation
    if v_start_time <= passed_time <= v_end_time:
      s[1] = viol_rate
      s[0] = 0
      l = 1 # violation

    if push_start_time <= passed_time <= push_end_time:
      s[1] = with_push
      s[0] = 1
      l = 0 # no violation

    step = random.randint(options.min_step, options.max_step)

    t = options.t + passed_time
    f_s.write("%s %s %s\n" % (str(t), str(s[0]), str(s[1])))
    f_l.write("%s %s\n" % (str(t), str(l)))

    if options.sleep:
      sys.sleep(step)
    passed_time += step

def gauss_signal_generator(f_s, f_l, options, gauss_options):
  passed_time = 0
  v_start_time = options.duration + 1
  v_label_time = options.duration + 1
  v_end_time = v_start_time

  if options.violate:
    v_dur = random.uniform(options.min_v_dur, options.max_v_dur)
    v_start_time = random.uniform(0, options.duration - v_dur)
    if v_dur >= options.v_dur:
      v_label_time = v_start_time + options.v_dur
    v_end_time = v_start_time + v_dur

  while passed_time < options.duration:

    s = random.gauss(gauss_options.mean, gauss_options.deviation)
    t = options.t + passed_time
    if v_start_time <= passed_time <= v_end_time:
      # s += gauss_options.v_diff
      s = random.gauss(gauss_options.mean +  gauss_options.v_diff, gauss_options.deviation)
    # additional noise:
    if random.uniform(0, 1) < 0.02:
      s = random.gauss(gauss_options.mean*3, gauss_options.deviation)
    s = min(s, options.s_max)
    s = max(s, options.s_min)
    f_s.write("%s %s\n" % (str(t), str(s)))
    if  v_label_time <= passed_time <= v_end_time:
      f_l.write("%s 1\n" % str(t))
    else:
      f_l.write("%s 0\n" % str(t))

    step = random.randint(options.min_step, options.max_step)
    if options.sleep:
      sys.sleep(step)
    passed_time += step






