import unittest
import formula_generator as fg
import formula_utilities as U


class MyTestCase(unittest.TestCase):

  def test_generate_formula_tree_op0(self):

    tree_list = fg.generate_formula_tree('x', 0, False)
    expected_formula_list = ['p < px', 'p > px']
    computed_list = [tree.to_formula() for tree in tree_list]
    self.assertSetEqual(set(expected_formula_list), set(computed_list))

  def test_generate_formula_tree_op1(self):
    tree_list = fg.generate_formula_tree('x', 1, False)
    expected_formula_list = ['! ( p < px )', '! ( p > px )',
                             '( p < px ) & ( p < px )', '( p < px ) & ( p > px )',
                             '( p > px ) & ( p < px )', '( p > px ) & ( p > px )',
                             '( p < px ) | ( p < px )', '( p < px ) | ( p > px )',
                             '( p > px ) | ( p < px )', '( p > px ) | ( p > px )',
                             'P 0 pP ( p < px )', 'P 0 pP ( p > px )',
                             'A 0 pA ( p < px )', 'A 0 pA ( p > px )',
                             '( p < px ) S 0 pS ( p < px )', '( p < px ) S 0 pS ( p > px )',
                             '( p > px ) S 0 pS ( p < px )', '( p > px ) S 0 pS ( p > px )']
    computed_list = [tree.to_formula() for tree in tree_list]
    self.assertSetEqual(set(expected_formula_list), set(computed_list))


  def test_generate_formula_tree_op2(self):
    tree_list = fg.generate_formula_tree('x', 2, False)
    count_with_0_operator = 2
    count_with_1_operator = 18
    count_with_2_root_binary_operator = 3*2*count_with_0_operator*count_with_1_operator
    count_with_2_root_unary_operator = 3*count_with_1_operator

    expected_formula_count = count_with_2_root_binary_operator + count_with_2_root_unary_operator
    self.assertEqual(len(tree_list), expected_formula_count)

    # A few scheck:
    formulas = set([ "".join(tree.to_formula().split())  for tree in tree_list])

    self.assertIn("".join('A 0 pA ( P 0 pP (p < px) )'.split()), formulas)

  def _test_generate_base_formulas(self):
    r = fg.generate_base_formulas( ['x'], {'x':0})

    self.assertIn('x > px0', r)
    self.assertIn('x < px1', r)


  def _test_generate_base_formulas2(self):
    r = fg.generate_base_formulas( ['x', 'y'], {'x':0, 'y':0})

    self.assertIn('x > px0', r)
    self.assertIn('x < px1', r)
    self.assertIn('y > py0', r)
    self.assertIn('y < py1', r)

  def _test_generage_formula(self):
    metric_list = ['x']
    indices = { x:0 for x in metric_list+U.STLPastOperators }

    r = fg.generate(depth=0, width=1, metric_list=metric_list, indices=indices)

    self.assertIn('x > px0', r)
    self.assertIn('x < px1', r)
    # generate(depth, width, metric_list, metric_limits, indices):

  def _test_generage_formula_width2(self):
    metric_list = ['x']
    indices = { x:0 for x in metric_list+U.STLPastOperators }

    r = fg.generate(depth=0, width=2, metric_list=metric_list, indices=indices)

    self.assertIn('x > px0 & x > px2', r)
    self.assertIn('x < px1 & x > px2', r)
    self.assertIn('x > px0 & x < px3', r)
    self.assertIn('x < px1 & x < px3', r)
    self.assertIn('x > px0 | x > px2', r)
    self.assertIn('x < px1 | x > px2', r)
    self.assertIn('x > px0 | x < px3', r)
    self.assertIn('x < px1 | x < px3', r)

    self.assertIn('! x > px2', r)
    self.assertIn('! x < px3', r)


  def _test_generage_formula_depth1(self):
    metric_list = ['x']
    indices = { x:0 for x in metric_list+U.STLPastOperators }

    r = fg.generate(depth=1, width=1, metric_list=metric_list, indices=indices)

    self.assertIn('x > px0 S 0 pS0 x > px2', r)
    self.assertIn('x < px1 & x > px2', r)
    self.assertIn('x > px0 & x < px3', r)
    self.assertIn('x < px1 & x < px3', r)
    self.assertIn('x > px0 | x > px2', r)
    self.assertIn('x < px1 | x > px2', r)
    self.assertIn('x > px0 | x < px3', r)
    self.assertIn('x < px1 | x < px3', r)

    self.assertIn('! x > px2', r)
    self.assertIn('! x < px3', r)
if __name__ == '__main__':
  unittest.main()
