

import formula_utilities as U
import itertools
import STL


# See tests for the details of this class.
# It might make more sense to generate all possible formulas using SyntaxTrees.
# first, test that approach...

def generate_base_formulas(metric_list, indices):
  return_list = []
  for m in metric_list:
    for op in U.MetricOperators:
      parameter = 'p' + m + str(indices[m])
      indices[m] += 1
      return_list.append( m + ' ' + op + ' ' + parameter)
  return return_list


def generate(depth, width, metric_list, indices):
  """

  Args:
    depth: The number of nested temporal operators.
    width: The number of boolean operators that can be used in each level.
    metric_list:
    indices:

  Returns: A list of tuples in the following form: [(formula, [list of parameter names])].
  """

  if depth == 0: # Base case, no temporal operator.
    if width == 1: # Base case for the width.
      return generate_base_formulas(metric_list, indices)
    else:
      left_formulas = generate(depth, 1, metric_list, indices)
      right_formulas = generate(depth, width-1, metric_list, indices)
      formulas = []
      for lf, rf in itertools.product(*[left_formulas, right_formulas]):
        for op in U.BooleanOperators:
          if U.is_binary(op):
            formulas.append(lf + ' ' + op + ' ' + rf)
          else:
            formulas.append(op + ' ' + rf)

      return formulas

  else:
    # First reduce depth.
    left_formulas = generate(depth-1, width, metric_list, indices)
    right_formulas = generate(depth-1, width, metric_list, indices)
    formulas = []
    for lf, rf in itertools.product(*[left_formulas, right_formulas]):
      for op in U.STLPastOperators:
        op_with_parameter = op + ' 0 p' + op + str(indices[op])
        indices[op] += 1
        if U.is_binary(op):
          formulas.append(lf + ' ' + op_with_parameter + ' ' +  rf)
        else:
          formulas.append(op_with_parameter + ' ' + rf)

    return formulas




def generate_formula_tree(metric_list, operator_count, return_formula_string):

  return_list = []
  if operator_count == 0:
    for op in U.MetricOperators:
      for m in metric_list:
        stn = STL.SyntaxTreeNode()
        stn.op = op
        stn.param = 'p' + str(m)
        stn.metric = m
        return_list.append(stn)

    if return_formula_string:
      return [t.to_formula() for t in return_list]
    return return_list

  # Compute all with less number of operators:
  all_trees = {}
  for oc in xrange(operator_count):
    all_trees[oc] = generate_formula_tree(metric_list, oc, False)

  # now combine them
  for op in U.BooleanOperators + U.STLPastOperators:

    if U.is_binary(op):
      # For symmetric operators like and/or, only go through the half
      lc_count = operator_count
      if U.is_boolean(op):
        lc_count = 1 + operator_count/2
      for lc in xrange(lc_count):
        for lf in all_trees[lc]:
          for rf in all_trees[operator_count - lc - 1]:
            stn = STL.SyntaxTreeNode()
            stn.op = op
            stn.left_node = lf
            stn.right_node = rf
            return_list.append(stn)

    else:
      for lf in all_trees[operator_count - 1]:
        stn = STL.SyntaxTreeNode()
        stn.op = op
        stn.left_node = lf
        return_list.append(stn)

  if return_formula_string:
    return [t.to_formula() for t in return_list]
  return return_list







